-- Copyright (c) 2006 Frank Buss (fb@frank-buss.de)
-- See license.txt for license
--
-- Test-bench for the DS2432
--
-- This simulates the DS2432 response for testing the ds2432 entity.

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity ds2432_test is
end entity ds2432_test;

architecture test of ds2432_test is
  constant system_speed: natural := 50e6;
  constant rom_test: unsigned(63 downto 0) := x"cf000000e4ec2933";

  signal ds_wire_rom: unsigned(47 downto 0) := (others => '0');
  signal ds_wire: std_logic;
  signal clk: std_logic := '0';
  signal data_valid: std_logic := '0';

begin
  
  ds2432_inst: entity ds2432
    generic map(system_speed)
    port map(
      clk_i => clk,
      rst_i => '0',
      rom_id_o => ds_wire_rom,
      data_valid_o => data_valid,
      wire_io => ds_wire);

  test_loop: process
    variable rom: unsigned(63 downto 0);
  begin
    -- init wire
    ds_wire <= 'Z';
    
    -- wait for reset
    wait until ds_wire = '0';
    wait until ds_wire = 'Z';
    wait for 30 us;
    ds_wire <= '0';
    wait for 100 us;
    ds_wire <= 'Z';
                        
    -- wait until first byte is written
    for j in 1 to 8 loop
      wait until ds_wire = '0';
      wait until ds_wire = 'Z';
    end loop;

    -- simulate ROM result
    rom := rom_test;
    for j in 1 to 64 loop
      wait until ds_wire = '0';
      wait until ds_wire = 'Z';
      if rom(0) = '0' then
        ds_wire <= '0';
        wait for 30 us;
        ds_wire <= 'Z';
        wait for 1 us;
      end if;
      rom := shift_right(rom, 1);
    end loop;
    
    -- wait until latched
    wait until data_valid = '1';

    -- check, if read process was successful
    assert rom_test(55 downto 8) = ds_wire_rom report "ROM read error" severity failure;

    -- stop simulation
    assert false report "No failure, simulation was successful." severity failure; 

  end process;
  
  update_clock: process
  begin
    while true loop 
      clk <= not clk;
      wait for 10 ns;
    end loop;
  end process;

end architecture test;
